@@ -2,13 +2,14 @@ import asyncio
 import io
 import json
 import pathlib
-import zlib
+import sys
 from unittest import mock
 
 import pytest
 
 import aiohttp
 from aiohttp import payload
+from aiohttp.compression_utils import ZLibBackend
 from aiohttp.hdrs import (
     CONTENT_DISPOSITION,
     CONTENT_ENCODING,
@@ -18,7 +19,6 @@ from aiohttp.hdrs import (
 from aiohttp.helpers import parse_mimetype
 from aiohttp.multipart import MultipartResponseWrapper
 from aiohttp.streams import StreamReader
-from aiohttp.test_utils import make_mocked_coro
 
 BOUNDARY = b"--:"
 
@@ -96,21 +96,21 @@ class TestMultipartResponseWrapper:
 
     async def test_next(self) -> None:
         wrapper = MultipartResponseWrapper(mock.Mock(), mock.Mock())
-        wrapper.stream.next = make_mocked_coro(b"")
+        wrapper.stream.next = mock.AsyncMock(b"")
         wrapper.stream.at_eof.return_value = False
         await wrapper.next()
         assert wrapper.stream.next.called
 
     async def test_release(self) -> None:
         wrapper = MultipartResponseWrapper(mock.Mock(), mock.Mock())
-        wrapper.resp.release = make_mocked_coro(None)
+        wrapper.resp.release = mock.AsyncMock(None)
         await wrapper.release()
         assert wrapper.resp.release.called
 
     async def test_release_when_stream_at_eof(self) -> None:
         wrapper = MultipartResponseWrapper(mock.Mock(), mock.Mock())
-        wrapper.resp.release = make_mocked_coro(None)
-        wrapper.stream.next = make_mocked_coro(b"")
+        wrapper.resp.release = mock.AsyncMock(None)
+        wrapper.stream.next = mock.AsyncMock(b"")
         wrapper.stream.at_eof.return_value = True
         await wrapper.next()
         assert wrapper.stream.next.called
@@ -377,6 +377,17 @@ class TestPartReader:
         )
         assert result == expected
 
+    async def test_decode_with_content_transfer_encoding_base64(self) -> None:
+        with Stream(b"VG\r\r\nltZSB0byBSZ\r\nWxheCE=\r\n--:--") as stream:
+            obj = aiohttp.BodyPartReader(
+                BOUNDARY, {CONTENT_TRANSFER_ENCODING: "base64"}, stream
+            )
+            result = b""
+            while not obj.at_eof():
+                chunk = await obj.read_chunk(size=6)
+                result += obj.decode(chunk)
+        assert b"Time to Relax!" == result
+
     @pytest.mark.parametrize("encoding", ("binary", "8bit", "7bit"))
     async def test_read_with_content_transfer_encoding_binary(
         self, encoding: str
@@ -451,7 +462,7 @@ class TestPartReader:
         assert data == result
 
     async def test_read_text_compressed(self) -> None:
-        data = b"\x0b\xc9\xccMU(\xc9W\x08J\xcdI\xacP\x04\x00" b"%s--:--" % newline
+        data = b"\x0b\xc9\xccMU(\xc9W\x08J\xcdI\xacP\x04\x00%s--:--" % newline
         with Stream(data) as stream:
             obj = aiohttp.BodyPartReader(
                 BOUNDARY,
@@ -503,9 +514,7 @@ class TestPartReader:
         assert {"тест": "пассед"} == result
 
     async def test_read_json_compressed(self) -> None:
-        with Stream(
-            b"\xabV*I-.Q\xb2RP*H,.NMQ\xaa\x05\x00" b"%s--:--" % newline
-        ) as stream:
+        with Stream(b"\xabV*I-.Q\xb2RP*H,.NMQ\xaa\x05\x00%s--:--" % newline) as stream:
             obj = aiohttp.BodyPartReader(
                 BOUNDARY,
                 {CONTENT_ENCODING: "deflate", CONTENT_TYPE: "application/json"},
@@ -700,7 +709,7 @@ class TestMultipartReader:
                     b"----:--",
                     b"",
                     b"passed",
-                    b"----:----" b"--:--",
+                    b"----:------:--",
                 ]
             )
         ) as stream:
@@ -754,6 +763,66 @@ class TestMultipartReader:
             with pytest.raises(ValueError):
                 await reader.next()
 
+    @pytest.mark.skipif(sys.version_info < (3, 10), reason="Needs anext()")
+    async def test_read_boundary_across_chunks(self) -> None:
+        class SplitBoundaryStream:
+            def __init__(self) -> None:
+                self.content = [
+                    b"--foobar\r\n\r\n",
+                    b"Hello,\r\n-",
+                    b"-fo",
+                    b"ob",
+                    b"ar\r\n",
+                    b"\r\nwor",
+                    b"ld!",
+                    b"\r\n--f",
+                    b"oobar--",
+                ]
+
+            async def read(self, size=None) -> bytes:
+                chunk = self.content.pop(0)
+                assert len(chunk) <= size
+                return chunk
+
+            def at_eof(self) -> bool:
+                return not self.content
+
+            async def readline(self) -> bytes:
+                line = b""
+                while self.content and b"\n" not in line:
+                    line += self.content.pop(0)
+                line, *extra = line.split(b"\n", maxsplit=1)
+                if extra and extra[0]:
+                    self.content.insert(0, extra[0])
+                return line + b"\n"
+
+            def unread_data(self, data: bytes) -> None:
+                if self.content:
+                    self.content[0] = data + self.content[0]
+                else:
+                    self.content.append(data)
+
+        stream = SplitBoundaryStream()
+        reader = aiohttp.MultipartReader(
+            {CONTENT_TYPE: 'multipart/related;boundary="foobar"'}, stream
+        )
+        part = await anext(reader)
+        result = await part.read_chunk(10)
+        assert result == b"Hello,"
+        result = await part.read_chunk(10)
+        assert result == b""
+        assert part.at_eof()
+
+        part = await anext(reader)
+        result = await part.read_chunk(10)
+        assert result == b"world!"
+        result = await part.read_chunk(10)
+        assert result == b""
+        assert part.at_eof()
+
+        with pytest.raises(StopAsyncIteration):
+            await anext(reader)
+
     async def test_release(self) -> None:
         with Stream(
             newline.join(
@@ -1120,6 +1189,7 @@ async def test_writer_write_no_parts(buf, stream, writer) -> None:
     assert b"--:--\r\n" == bytes(buf)
 
 
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_writer_serialize_with_content_encoding_gzip(buf, stream, writer):
     writer.append("Time to Relax!", {CONTENT_ENCODING: "gzip"})
     await writer.write(stream)
@@ -1130,7 +1200,7 @@ async def test_writer_serialize_with_content_encoding_gzip(buf, stream, writer):
         b"Content-Encoding: gzip" == headers
     )
 
-    decompressor = zlib.decompressobj(wbits=16 + zlib.MAX_WBITS)
+    decompressor = ZLibBackend.decompressobj(wbits=16 + ZLibBackend.MAX_WBITS)
     data = decompressor.decompress(message.split(b"\r\n")[0])
     data += decompressor.flush()
     assert b"Time to Relax!" == data
